Option Strict Off
Option Explicit On
Friend Class clsGMST
	'(C)Roy E. Hoffman 2000
    'See Main.bas for terms and conditions of use
    'Handles GMST (Greenwich mean sidreal time).
    'Midday is when the point of Aries is at the meridian a zero longitude.
	
	'Some of the algorithms from
	'J. Meeus, "Astronomical Algorithms", 2nd Ed., William-Bell Inc., Richmond, VG, 1998, p. 87
    Friend Function FromGMST(ByVal udtTime As Mytime, ByRef blnValid As Boolean, ByVal enmType As Main.DateType) As Double
        'Convert from GMST to a fraction of a day
        Dim cDMS As New clsDMS
        Dim dblT As Double 'Date in Julian centuries from J2000
        Dim dblT0 As Double 'Time in days
        Dim cHebrew As New clsHebrew

        On Error GoTo EH
        blnValid = True
        'Julian centuries from 2000
        dblT = (Int(dblDate - 2451545) + 1721424.5) / 36525 'Julian centuries at 0 UTC
        'Time at 0 UTC
        dblT0 = (6.697374558 + dblT * (2400.0513369072 + dblT * (0.000025862 - dblT * 0.00000000172))) / 24
        dblT0 = dblT0 - Int(dblT0) 'Fraction of day
        FromGMST = cDMS.FromDMS(udtTime.udtDms, blnValid) / 24 - dblT0 'Time
        If Not blnValid Then Exit Function
        'Place within the day
        If udtTime.blnSecondTime Then FromGMST = FromGMST + 1
        FromGMST = (FromGMST - Int(FromGMST)) * 0.99726956633
        If udtTime.blnSecondTime Then FromGMST = FromGMST + 0.99726956633
        If FromGMST > 1 And enmType < Main.DateType.Hebrew Then
            blnValid = False
        ElseIf enmType >= Main.DateType.Hebrew Then
            Call cHebrew.AdjustTime(Int(dblDate), FromGMST, blnValid, udtTime.blnSecondTime)
        End If
        Exit Function
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Function
    Friend Sub ToGMST(ByVal dblFix As Double, ByRef udtTimeIn As Mytime, ByRef blnValid As Boolean)
        'convert a fixed radix to GMST
        Dim cDMS As New clsDMS
        Dim cSun As New clsSun
        Dim cUTC As New clsUTC
        Dim dblT As Double 'Date in Julian centuries from J2000
        Dim dblT0 As Double 'Time in days
        Dim dblTime2 As Double
        Dim dblStart As Double
        Dim dblEnd As Double
        Dim intDayShift As Short

        On Error GoTo EH
        blnValid = True
        If System.Math.Abs(dblFix) > FixMax Then
            Err.Raise(vbObjectError + 1, "cGMST.ToGMST", "Magnitude of dblDate too large: " & dblFix)
        End If
        udtTimeIn.bln24Hr = True
        udtTimeIn.enmTimeType = Main.TimeType.gmst
        dblT = (Int(dblFix - 2451545) + 1721424.5) / 36525 'Julian centuries since J2000 at 0 UT
        'GMST at 0 UT
        dblT0 = (6.697374558 + dblT * (2400.0513369072 + dblT * (0.000025862 - dblT * 0.00000000172))) / 24
        'Place within the day
        Call cUTC.StartEndOfDay(dblFix, udtDate.enmDateType, dblStart, dblEnd, blnValid, intDayShift)
        If Not blnValid Then Exit Sub
        dblT0 = dblT0 - Int(dblT0)
        udtTime.blnSecondTime = (dblFix - dblStart) / 0.99726956633 >= 1 'Check for 2nd occurence
        dblTime2 = (dblFix - Int(dblFix)) / 0.99726956633 + dblT0
        dblTime2 = dblTime2 - Int(dblTime2)
        Call cDMS.ToDMS(dblTime2 * 24, udtTimeIn.udtDms) 'Convert to time in base 60
        If udtTimeIn.udtDms.intDegreesHours > 23 Then
            Err.Raise(vbObjectError + 1, "cGMST.ToGMST", "Invalid udtTimeIn.udtDms.intDegreesHours: " & udtTimeIn.udtDms.intDegreesHours)
        End If
        Exit Sub
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Sub
    Friend Function FixToGMST(ByVal dblFix As Double, ByVal blnTD As Boolean, ByRef blnValid As Boolean) As Double 'In degrees
        'dblFix is fixed radix in UTC or TD to GMST in degrees
        Dim dblT As Double
        Dim dblDateUTC As Double
        Dim cDeltaT As New clsDeltaT

        On Error GoTo EH
        blnValid = True
        If System.Math.Abs(dblFix) > FixMax + 2 Then
            'if converting to UTC then give a little more leaway and check at the end
            Err.Raise(vbObjectError + 1, "cLST.FixToLST", "Magnitude of dblFix too large: " & dblFix)
        End If
        If blnTD Then 'Convert from Terrestial time to UT
            FixToGMST = dblFix - cDeltaT.DeltaT(Int(dblFix), blnValid, False)
            If Not blnValid Then Exit Function
        Else 'Convert from UTC
            FixToGMST = dblFix
        End If
        dblT = (FixToGMST - 730120.5) / 36525 'Julian centuries since J2000
        FixToGMST = FracCirc(280.46061837 + 360.98564736629 * dblT * 36525 + dblT * dblT * (0.000387933 - dblT / 38710000))
        Exit Function
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Err.Raise(Err.Number)
        End If
    End Function
End Class