Option Strict Off
Option Explicit On
Friend Class clsHebrew
    '(C)Roy E. Hoffman 2000-2011
    'Deals with Hebrew calendar and Hebrew day functions
	'See Main.bas for terms and conditions of use
	'Most of the algorithms in this module are from
	'E. M. Reingold and N. Dershowitz "Calendrical Calculations: The Millenium Edition", Cambridge Univ. Press, Cambridge, UK, 1999.
	
    Const HebrewEpoch As Integer = -1373427
    Const YearMin As Integer = -(FixMax - 1373427) / 365.24682
    Const YearMax As Integer = (FixMax + 1373427) / 365.24682

    Friend Function IsFestival(ByVal lngDate As Integer) As Boolean
        'Determines if it is a day on which work is forbidden: Shabbat and festivals
        'Used to determine if a change between summer and winter time should be delayed
        'due to a festival according to Israeli law starting in 2011 (does not include
        '2nd day yom tov escept Rosh Hashannah)
        Dim udtDateIn As MyDate 'Date
        Dim blnValid As Boolean

        On Error GoTo EH
        blnValid = True
        'Convert to Hebrew date
        Call fixed_hebrew(lngDate, udtDateIn, blnValid)
        If Not blnValid Then
            Err.Raise(vbObjectError + 1, "cHebrew.IsFestival", "Invalid Date conversion")
        End If
        'Rosh Hashannah, Yom Kippur, Succot, Simchat Torah, Pesach, Shavuot, Shabbat
        IsFestival = (udtDateIn.intMonth = 7 And (udtDateIn.intDateOfMonth = 1 Or _
        udtDateIn.intDateOfMonth = 2 Or udtDateIn.intDateOfMonth = 10 Or _
        udtDateIn.intDateOfMonth = 15 Or udtDateIn.intDateOfMonth = 22) Or _
        udtDateIn.intMonth = 1 And (udtDateIn.intDateOfMonth = 15 Or _
        udtDateIn.intDateOfMonth = 21) Or udtDateIn.intMonth = 3 And _
        udtDateIn.intDateOfMonth = 6 Or MyMod(lngDate, 7) = 6)
        Exit Function
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Function
    Friend Function BeforeOnAfter(ByVal intDayOfWeek As Short, ByVal intDateOfMonth As Short, ByVal intWeekNo As Short, ByVal intMonth As Short, ByVal dblDate As Double, ByRef blnValid As Boolean) As Short
        'Returns -1 before, 0 on, 1 after
        'intWeekNo is the number of weeks after the date on the intDayOfWeek defined
        'intWeekNo 0 means on that date and intWeekNo 5 means the last week of the month
        'Used for determining if before on or after a summer to winter time change according
        'to the Hebrew calendar. This is according to Israeli law from 2003-2010.
        Dim udtDateIn As MyDate 'Date
        Dim udtDateGreg As MyDate 'Gregorian date
        Dim intDateNo As Short '
        Dim intDateNo2 As Short
        Dim cGregorian As New clsGregorian

        On Error GoTo EH
        blnValid = True
        If System.Math.Abs(dblDate) > FixMax + 2 Then
            Err.Raise(vbObjectError + 1, "cHebrew.BeforeOnAfter", "Magnitude of dblDate too large: " & dblDate)
        End If
        If intDayOfWeek < 0 Or intDayOfWeek > 6 Then
            Err.Raise(vbObjectError + 1, "cHebrew.BeforeOnAfter", "Invalid intDayOfWeek: " & intDayOfWeek)
        End If
        If intMonth < 1 Or intMonth > 13 Then
            Err.Raise(vbObjectError + 1, "cHebrew.BeforeOnAfter", "Invalid intMonth: " & intMonth)
        End If
        If (intDateOfMonth < 1 Or intDateOfMonth > 30) Then
            Err.Raise(vbObjectError + 1, "cHebrew.BeforeOnAfter", "Invalid intDateOfMonth: " & intDateOfMonth)
        End If
        If intWeekNo < 0 Or intWeekNo > 5 Then
            Err.Raise(vbObjectError + 1, "cHebrew.BeforeOnAfter", "Invalid intWeekNo: " & intWeekNo)
        End If
        Call fixed_hebrew(Int(dblDate), udtDateIn, blnValid)
        'Months
        Call cGregorian.fixed_gregorian(Int(dblDate), udtDateGreg, blnValid)
        If udtDateGreg.intMonth >= 11 Then 'Kludge it to stop summer time by accident if in Tevet and the conversion is in the wrong year
            BeforeOnAfter = 1
        Else
            If (intMonth + 1) Mod 12 > (udtDateIn.intMonth + 1) Mod 12 Then
                Call cGregorian.fixed_gregorian(Int(dblDate), udtDateIn, blnValid)
                If Not blnValid Then Exit Function
                If udtDateIn.intMonth >= 11 Then 'Kludge it to stop summer time by accident
                    BeforeOnAfter = 1
                Else
                    BeforeOnAfter = -1
                End If
            ElseIf (intMonth + 3) Mod 12 < (udtDateIn.intMonth + 3) Mod 12 Then
                BeforeOnAfter = 1
            Else 'If same month
                If intWeekNo = 0 Then
                    intDateNo = intDateOfMonth
                ElseIf intWeekNo = 5 Then  'Last week of month
                    intDateNo = LastDayOfHebrewMonth(udtDateIn.intMonth, udtDateIn.intYear, blnValid) - 6
                    If Not blnValid Then Exit Function
                Else 'Count the weeks from the date, 1 is 1st week
                    intDateNo = intDateOfMonth + 7 * (intWeekNo - 1)
                End If
                If intWeekNo > 0 Then 'Select the day
                    intDateNo2 = udtDateIn.intDateOfMonth
                    udtDateIn.intDateOfMonth = intDateNo
                    intDateNo = intDateNo + MyMod(intDayOfWeek - cGregorian.fixed_DayOfWeek(frmDate.EvaluateDate(udtDateIn, blnValid)), 7)
                    If Not blnValid Then Exit Function
                    udtDateIn.intDateOfMonth = intDateNo2
                End If
                If udtDateIn.intDateOfMonth > intDateNo Then
                    BeforeOnAfter = 1
                ElseIf udtDateIn.intDateOfMonth = intDateNo Then
                    BeforeOnAfter = 0
                Else 'Before the date
                    BeforeOnAfter = -1
                End If
            End If
        End If
        Exit Function
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Function
    Function HebrewLeapYear(ByVal intYear As Short) As Boolean
        'Is it a leap year
        'The leap year occurs 7 times during the 19 year cycle
        'On the 3rd, 6th, 8th, 11th, 14th, 17th and 19th year
        HebrewLeapYear = MyMod(7 * CInt(intYear) + 1, 19) < 7
    End Function
    Function MonthsInHebrewYear(ByVal intYear As Short) As Short
        'Returns the number of months in the year: 12 for a regular and 13 for a leap year
        If HebrewLeapYear(intYear) Then
            MonthsInHebrewYear = 13
        Else
            MonthsInHebrewYear = 12
        End If
    End Function
    Function HebrewElapsedDays(ByVal intYear As Short) As Integer
        'Number of days since the epoch at the start of the year
        'This does not take account of Gatrad or Btu-takpat
        Dim lngMonthsElapsed As Integer 'Months since epoch
        Dim lngPartsElapsed As Integer 'Remaining parts since epoch
        Dim lngHoursElapsed As Integer 'Remaining hours since epoch
        Dim lngDaysElapsed As Integer 'Days since epoch
        Static lastYear As Short = 0 'Store the last result
        Static lastResult As Integer = 0

        If intYear = lastYear And intYear <> 0 Then
            HebrewElapsedDays = lastResult
        Else
            lngMonthsElapsed = Int((235 * CInt(intYear) - 234) / 19)
            lngPartsElapsed = 204 + 793 * MyMod(lngMonthsElapsed, 1080)
            lngHoursElapsed = 11 + 12 * lngMonthsElapsed + 793 * Int(lngMonthsElapsed / 1080) + Int(lngPartsElapsed / 1080)
            lngDaysElapsed = 29 * lngMonthsElapsed + Int(lngHoursElapsed / 24)
            If MyMod(3 * (lngDaysElapsed + 1), 7) < 3 Then
                HebrewElapsedDays = lngDaysElapsed + 1 'Lo Adu Rosh
            Else
                HebrewElapsedDays = lngDaysElapsed
            End If
            lastYear = intYear
            lastResult = HebrewElapsedDays
        End If
    End Function
    Function HebrewNewYearDelay(ByVal intYear As Short) As Short
        'Calculates the dechiyot due to Gatrad and Btu-takpat
        Dim lngNY0 As Integer 'Last year
        Dim lngNY1 As Integer 'This year
        Dim lngNY2 As Integer 'Next year
        Static lastYear As Short = 0 'Store the last result
        Static lastResult As Short = 0

        If intYear = lastYear And intYear <> 0 Then
            HebrewNewYearDelay = lastResult
        Else
            lngNY0 = HebrewElapsedDays(intYear - 1)
            lngNY1 = HebrewElapsedDays(intYear)
            lngNY2 = HebrewElapsedDays(intYear + 1)
            If lngNY2 - lngNY1 = 356 Then 'Gatrad
                HebrewNewYearDelay = 2
            ElseIf lngNY1 - lngNY0 = 382 Then 'Btu takpat
                HebrewNewYearDelay = 1
            Else
                HebrewNewYearDelay = 0
            End If
            lastYear = intYear
            lastResult = HebrewNewYearDelay
        End If
    End Function
    Function HebrewNewYear(ByVal intYear As Short) As Integer
        'Returns the number of days elapsed till the New Year taking into account all the dechiyot
        HebrewNewYear = HebrewEpoch + HebrewElapsedDays(intYear) + HebrewNewYearDelay(intYear)
    End Function
    Function DaysInHebrewYear(ByVal intYear As Short) As Short
        'Returns the number of days in a Hebrew year
        Static lastYear As Short = 0 'Store the last result
        Static lastResult As Short = 0

        If intYear = lastYear And intYear <> 0 Then
            DaysInHebrewYear = lastResult
        Else
            DaysInHebrewYear = HebrewNewYear(intYear + 1) - HebrewNewYear(intYear)
            lastYear = intYear
            lastResult = DaysInHebrewYear
        End If
    End Function
    Function ShortKislev(ByVal intYear As Short) As Boolean
        'Does Kislev have 29 days?
        Static lastYear As Short = 0 'Store the last result
        Static lastResult As Short = 0

        If intYear = lastYear And intYear <> 0 Then
            ShortKislev = lastResult
        Else
            ShortKislev = (DaysInHebrewYear(intYear) Mod 10 = 3) '353 or 383 days
            lastYear = intYear
            lastResult = ShortKislev
        End If
    End Function
    Function LongCheshvan(ByVal intYear As Short) As Boolean
        'Does Cheshvan have 30 days?
        Static lastYear As Short = 0 'Store the last result
        Static lastResult As Short = 0

        If intYear = lastYear And intYear <> 0 Then
            LongCheshvan = lastResult
        Else
            LongCheshvan = (DaysInHebrewYear(intYear) Mod 10 = 5) '355 or 385 days
            lastYear = intYear
            lastResult = LongCheshvan
        End If
    End Function
    Function LastDayOfHebrewMonth(ByVal intMonth As Short, ByVal intYear As Short, ByRef blnValid As Boolean) As Short
        'Returns the last date of the month, 29 or 30
        Static lastMonth As Short = 1 'Store the last result
        Static lastYear As Short = 0
        Static lastResult As Short = 0

        On Error GoTo EH
        blnValid = True
        If intMonth < 1 Or intMonth > MonthsInHebrewYear(intYear) Then
            Err.Raise(vbObjectError + 1, "cHebrew.LastDayOfHebrewMonth", "Invalid intMonth: " & intMonth)
        End If
        If intMonth = lastMonth And intYear = lastYear And intYear <> 0 Then
            LastDayOfHebrewMonth = lastResult
        Else
            If intMonth = 2 Or intMonth = 4 Or intMonth = 6 Or intMonth = 10 Or intMonth = 13 Or intMonth = 12 And Not HebrewLeapYear(intYear) Or intMonth = 8 And Not LongCheshvan(intYear) Or intMonth = 9 And ShortKislev(intYear) Then
                'Iyar, Tammuz, Ellul, Shevat, Adar 2 and sometimes Cheshvan, Kislev and Adar
                LastDayOfHebrewMonth = 29
            Else
                LastDayOfHebrewMonth = 30
            End If
            lastMonth = intMonth
            lastYear = intYear
            lastResult = LastDayOfHebrewMonth
        End If
        Exit Function
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Function
    Friend Function hebrew_fixed(ByVal udtDate As MyDate, ByRef blnValid As Boolean) As Integer
        'Convert a Hebrew date to fixed radix
        Dim intM As Short 'Month counter
        Static lastDate As MyDate 'Store previous result
        Static lastResult As Double
        Dim udtDate2 As MyDate
        Dim cSun As New clsSun
        Dim dblSS As Double
        Dim intY As Integer

        On Error GoTo EH
        blnValid = True

        If udtDate.intDateOfMonth = lastDate.intDateOfMonth And udtDate.intMonth = lastDate.intMonth And udtDate.intYear = lastDate.intYear Then
            hebrew_fixed = lastResult
        Else
            If udtDate.intYear > YearMax Or -udtDate.intYear + 1 < YearMin And Not udtDate.blnCE Or udtDate.intYear < YearMin And udtDate.blnCE Then
                Err.Raise(vbObjectError + 1, "cHebrew.hebrew_fixed", "Magnitude of udtDate.intYear too large: " & udtDate.intYear)
            End If
            If udtDate.intMonth < 1 Or udtDate.intMonth > MonthsInHebrewYear(udtDate.intYear) Then
                Err.Raise(vbObjectError + 1, "cHebrew.hebrew_fixed", "Invalid udtDate.intMonth: " & udtDate.intMonth)
            End If
            If udtDate.intDateOfMonth < 1 Or udtDate.intDateOfMonth > LastDayOfHebrewMonth(udtDate.intMonth, udtDate.intYear, blnValid) Then
                Err.Raise(vbObjectError + 1, "cHebrew.hebrew_fixed", "Invalid udtDate.intDateOfMonth: " & udtDate.intDateOfMonth)
            End If
            If Not blnValid Then Exit Function
            If udtDate.blnCE Then 'Allow 'BCE'
                intY = udtDate.intYear
            Else
                intY = -udtDate.intYear + 1
            End If
            hebrew_fixed = HebrewNewYear(intY) + udtDate.intDateOfMonth - 1
            If udtDate.intMonth < 7 Then 'Add each month
                For intM = 7 To MonthsInHebrewYear(intY)
                    hebrew_fixed = hebrew_fixed + LastDayOfHebrewMonth(intM, intY, blnValid)
                Next
                For intM = 1 To udtDate.intMonth - 1
                    hebrew_fixed = hebrew_fixed + LastDayOfHebrewMonth(intM, intY, blnValid)
                Next
            Else
                For intM = 7 To udtDate.intMonth - 1
                    hebrew_fixed = hebrew_fixed + LastDayOfHebrewMonth(intM, intY, blnValid)
                Next
            End If
            If System.Math.Abs(hebrew_fixed) > FixMax + 2 Then
                Err.Raise(vbObjectError + 1, "cHebrew.hebrew_fixed", "Magnitude of hebrew_fixed too large: " & hebrew_fixed)
            End If
            lastDate.intDateOfMonth = udtDate.intDateOfMonth
            lastDate.intMonth = udtDate.intMonth
            lastDate.intYear = udtDate.intYear
            lastDate.blnCE = udtDate.blnCE
            lastResult = hebrew_fixed
        End If
        Exit Function
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Function
    Friend Sub fixed_hebrew(ByVal lngDate As Integer, ByRef udtDate As MyDate, ByRef blnValid As Boolean)
        'Convert a fixed radix date to the Hebrew Calendar
        Dim lngApprox As Double '1st approximation to year
        Static lastDate As Integer = 0 'Store previous result
        Static lastudtDate As MyDate
        Dim cSun As New clsSun
        Dim udtTimeZero As Mytime

        udtDate.enmDateType = Main.DateType.Hebrew
        udtTimeZero.enmTimeType = Main.TimeType.meansolar
        udtTimeZero.udtDms.intDegreesHours = 6
        udtTimeZero.udtDms.intMinutes = 0
        udtTimeZero.udtDms.dblSeconds = 0
        If lngDate = lastDate And lastDate <> 0 Then
            udtDate.intDateOfMonth = lastudtDate.intDateOfMonth
            udtDate.intMonth = lastudtDate.intMonth
            udtDate.intYear = lastudtDate.intYear
            udtDate.blnCE = lastudtDate.blnCE
        Else
            'Calculate the first approximation to the year
            lngApprox = Int((lngDate - HebrewEpoch) * 98496.0# / 35975351) + 1
            If System.Math.Abs(CInt(lngApprox > 32765)) Then
                blnValid = False
            Else
                'Find the radix date of the start of the year
                udtDate.intMonth = 7
                udtDate.intDateOfMonth = 1
                udtDate.intYear = lngApprox - 1
                udtDate.blnCE = True
                'Adjust the year if necessary
                While hebrew_fixed(udtDate, blnValid) <= lngDate
                    udtDate.intYear = udtDate.intYear + 1
                End While
                'Chech if after Nisan
                udtDate.intYear = udtDate.intYear - 1
                udtDate.intMonth = 1
                If lngDate < hebrew_fixed(udtDate, blnValid) Then
                    udtDate.intMonth = 7
                Else
                    udtDate.intMonth = 1
                End If
                'Now check each month
                udtDate.intDateOfMonth = LastDayOfHebrewMonth(udtDate.intMonth, udtDate.intYear, blnValid)
                While lngDate > hebrew_fixed(udtDate, blnValid)
                    udtDate.intMonth = udtDate.intMonth + 1
                    udtDate.intDateOfMonth = LastDayOfHebrewMonth(udtDate.intMonth, udtDate.intYear, blnValid)
                End While
                'Determine the date of the month
                udtDate.intDateOfMonth = 1
                udtDate.intDateOfMonth = lngDate - hebrew_fixed(udtDate, blnValid) + 1
                lastDate = lngDate
                lastudtDate.intDateOfMonth = udtDate.intDateOfMonth
                lastudtDate.intMonth = udtDate.intMonth
                lastudtDate.intYear = udtDate.intYear
            End If
        End If
    End Sub
    Friend Function MonthName_Renamed(ByVal intMonth As Short, Optional ByVal intYear As Short = -32767) As String
        'Returns the name of the Hebrew Month
        Static lastMonth As Short = 1 'Store previous result
        Static lastYear As Short = 0
        Static lastResult As String = ""

        On Error GoTo EH
        If intMonth = lastMonth And intYear = lastYear Then
            MonthName_Renamed = lastResult
        Else
            If intMonth <> 12 Or intYear = -32767 Then 'Names of months with Adar listed as Adar (1)
                MonthName_Renamed = strTexts(intfrmDateBase + (29 + intMonth) * intNLanguages + intLanguage)
            ElseIf HebrewLeapYear(intYear) Then 'List Adar 1 and Adar 2
                MonthName_Renamed = strTexts(intfrmDateBase + 43 * intNLanguages + intLanguage)
            Else 'List Adar
                MonthName_Renamed = strTexts(intfrmDateBase + 44 * intNLanguages + intLanguage)
            End If
            'Select Case intMonth
            '    Case 1
            'MonthName_Renamed = "Nisan"
            '    Case 2
            'MonthName_Renamed = "Iyar"
            '    Case 3
            'MonthName_Renamed = "Sivan"
            '    Case 4
            'MonthName_Renamed = "Tammuz"
            '    Case 5
            'MonthName_Renamed = "Av"
            '    Case 6
            'MonthName_Renamed = "Ellul"
            '    Case 7
            'MonthName_Renamed = "Tishri"
            '    Case 8
            'MonthName_Renamed = "Cheshvan"
            '    Case 9
            'MonthName_Renamed = "Kislev"
            '    Case 10
            'MonthName_Renamed = "Tevet"
            '    Case 11
            'MonthName_Renamed = "Shevat"
            '    Case 12
            'If intYear = -32767 Then 'Year not defined
            'MonthName_Renamed = "Adar (1)"
            'ElseIf HebrewLeapYear(intYear) Then
            'MonthName_Renamed = "Adar 1"
            'Else
            'MonthName_Renamed = "Adar"
            'End If
            '    Case 13
            'MonthName_Renamed = "Adar 2"
            lastMonth = intMonth
            lastYear = intYear
            lastResult = MonthName_Renamed
        End If
        Exit Function
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Function
    Friend Sub StartEndDay(ByVal dblFix As Double, ByRef dblStart As Double, ByRef dblEnd As Double, ByRef blnValid As Boolean, ByRef intDayShift As Short)
        'Get start and end of Hebrew or Islamic day for a particular time
        'and provide an adjustment for the number of days from the UTC day
        Dim blnNeverRises As Boolean 'Sun does not rise
        Dim blnNeverSets As Boolean 'Sun is always above the horizon
        Dim cHBody As New clsHBody
        Dim cLocal As clsLocalTime

        On Error GoTo EH
        blnValid = True
        If System.Math.Abs(dblFix) > FixMax Then
            Err.Raise(vbObjectError + 1, "cHebrew.StartEndDay", "Magnitude of dblFix too large: " & dblFix)
        End If
        'Calculate the previous and current sunset
        Call cHBody.RiseSet("Sun", Int(dblFix) - 1, True, dblStart, True, blnNeverSets, blnNeverRises, blnValid, 0, False, 90)
        Call cHBody.RiseSet("Sun", Int(dblFix), True, dblEnd, True, blnNeverSets, blnNeverRises, blnValid, 0, False, 90)
        If dblEnd < dblStart Or dblEnd > dblStart + 2 Then
            Err.Raise(vbObjectError + 1, "cHebrew.StartEndDay", "Difference between consecutive sunsets out of range: " & dblEnd & " - " & dblStart & " = " & dblEnd - dblStart)
        End If
        'Adjust to the correct day
        intDayShift = 0
        While dblStart > dblFix 'While the start is after the moment in time (dblFix)
            intDayShift = intDayShift - 1 'Go back a day
            dblEnd = dblStart
            Call cHBody.RiseSet("Sun", Int(dblFix) - 1 + intDayShift, True, dblStart, True, blnNeverSets, blnNeverRises, blnValid, 0, False, 90)
            If dblEnd < dblStart Or dblEnd > dblStart + 2 Then
                Err.Raise(vbObjectError + 1, "cHebrew.StartEndDay", "Difference between consecutive sunsets out of range: " & dblEnd & " - " & dblStart & " = " & dblEnd - dblStart)
            End If
        End While
        While dblEnd < dblFix 'While the end is before the moment in time (dblFix)
            intDayShift = intDayShift + 1 'Go forward a day
            dblStart = dblEnd
            Call cHBody.RiseSet("Sun", Int(dblFix) + intDayShift, True, dblEnd, True, blnNeverSets, blnNeverRises, blnValid, 0, False, 90)
            If dblEnd < dblStart Or dblEnd > dblStart + 2 Then
                Err.Raise(vbObjectError + 1, "cHebrew.StartEndDay", "Difference between consecutive sunsets out of range: " & dblEnd & " - " & dblStart & " = " & dblEnd - dblStart)
            End If
        End While
        If blnValid And (dblEnd < dblStart Or dblEnd > dblStart + 2.002) Then
            Err.Raise(vbObjectError + 1, "cHebrew.StartEndDay", "Difference between dblEnd and dblStart out of range: " & dblEnd & " - " & dblStart & " = " & dblEnd - dblStart)
        End If
        If blnValid And (dblStart > dblFix Or dblStart < dblFix - 1.3) Then
            Err.Raise(vbObjectError + 1, "cHebrew.StartEndDay", "Difference between dblFix and dblStart out of range: " & dblFix & " - " & dblStart & " = " & dblFix - dblStart)
        End If
        If blnValid And (dblEnd < dblFix Or dblEnd > dblFix + 1.3) Then
            Err.Raise(vbObjectError + 1, "cHebrew.StartEndDay", "Difference between dblEnd and dblFix out of range: " & dblEnd & " - " & dblFix & " = " & dblEnd - dblFix)
        End If
        If blnValid And System.Math.Abs(intDayShift) > 1 Then
            Err.Raise(vbObjectError + 1, "cHebrew.StartEndDay", "intDayShift out of range: " & intDayShift)
        End If
        Exit Sub
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Sub
    Friend Sub AdjustTime(ByVal lngDate As Integer, ByRef dblTime As Double, ByRef blnValid As Boolean, ByRef blnSecondTime As Boolean)
        'Adjust the supplied time for a Hebrew or Islamic date
        Dim intDayShift As Short
        Dim dblSet As Double
        Dim blnNeverRises As Boolean
        Dim blnNeverSets As Boolean
        Dim cHBody As New clsHBody

        On Error GoTo EH
        blnValid = True
        intDayShift = 0
        'If before sunset the local previous day then add a day
        Call cHBody.RiseSet("Sun", lngDate - 1, True, dblSet, True, blnNeverRises, blnNeverSets, blnValid)
        If Not blnValid Then Exit Sub
        While dblTime < dblSet - lngDate
            dblTime = dblTime + 1
            intDayShift = intDayShift + 1
            Call cHBody.RiseSet("Sun", lngDate - 1 - intDayShift, True, dblSet, True, blnNeverRises, blnNeverSets, blnValid)
            If Not blnValid Then Exit Sub
        End While
        'If after sunset today, subtract a day
        Call cHBody.RiseSet("Sun", lngDate, True, dblSet, True, blnNeverRises, blnNeverSets, blnValid)
        If Not blnValid Then Exit Sub
        While dblTime > dblSet - lngDate + 1 ' And Not blnSecondTime
            dblTime = dblTime - 1
            If blnSecondTime Then intDayShift = intDayShift - 1
            Call cHBody.RiseSet("Sun", lngDate - 1 - intDayShift, True, dblSet, True, blnNeverRises, blnNeverSets, blnValid)
            If Not blnValid Then Exit Sub
            'If blnSecondTime Then intDayShift = intDayShift - 1
        End While
        Call cHBody.RiseSet("Sun", lngDate + intDayShift, True, dblSet, True, blnNeverRises, blnNeverSets, blnValid)
        If Not blnValid Then Exit Sub
        If dblTime > dblSet - lngDate - intDayShift Then
            'This needs fixing along with frmMain.FixedToTime
            'Err.Raise(vbObjectError + 1, "cHebrew.AdjustTime", "Time " & dblTime & " after sunset: " & dblSet - lngDate - intDayShift)
        End If
        Exit Sub
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Sub
End Class