Option Strict Off
Option Explicit On
Friend Class clsSky
    'Deals with the brightness and extinction of the sky
    Const TwoPi As Double = 6.28318530717959
    Friend Function DeltaM(ByVal dblFix As Double, ByVal dblAlt As Double, ByVal dblAz As Double, ByVal intMode As Short, ByRef blnValid As Boolean) As Double
        'Calculate the extinction of stellar magnitude
        'intMode refers to the model used: 0 - Schaefer 1993
        '                                  1 - B. E. Schaefer, Sky Telesc., May 1998, 57
        '                                  2 - Schaefer 1993 without refraction correction for ozone
        Dim dblKR As Double 'Rayleigh extinction coefficient
        Dim dblKOz As Double 'Ozone extinction coefficient
        Dim dblKA As Double 'Aerosol extinction coefficient
        Dim dblKW As Double 'Water extinction coefficient
        Dim dblXG As Double 'Path length for gas (Reyleigh and water)
        Dim dblXOz As Double 'Path length for ozone
        Dim dblXA As Double 'Path length for aerosol
        Dim dblZ As Double 'azimuth angle (90 - dblAlt)
        Dim dblZR As Double 'azimuth angle corrected for refraction
        Dim udtSun As HBody 'Sun's parameters
        Dim blnRefValid As Boolean 'Refraction valid if not too far below the horizon
        Dim cHBody As New clsHBody
        Dim udtWeatherNow As WeatherNow

        On Error GoTo EH
        blnValid = True
        'Find position of Sun
        Call cHBody.TrueHBody("Sun", dblFix, udtSun, blnValid)
        If Not blnValid Then Exit Function
        dblZ = 90 - dblAlt
        'Model the current weather
        Call frmWeather.GetWeatherNow(dblFix, udtWeatherNow, blnValid)
        If Not blnValid Then Exit Function
        dblZR = 90 - dblAlt - cHBody.Refract(dblAlt, True, udtWeatherNow, blnRefValid, blnValid)
        If Not blnRefValid Then dblZR = dblZ
        'Calculate the extinction coefficients
        Call GetK(dblFix, intMode, udtSun.RATopo, udtSun.HourAngTopo, dblKA, dblKOz, dblKR, dblKW, dblAlt, blnValid)
        If Not blnValid Then Exit Function
        'Calculate the path lengths in atmospheres
        Call GetX(dblFix, intMode, dblZ, dblZR, dblXA, dblXOz, dblXG, blnValid)
        If Not blnValid Then Exit Function
        'Calculate the extinctions in magnitudes
        DeltaM = (dblKR + dblKW) * dblXG + dblKOz * dblXOz + dblKA * dblXA
        If DeltaM < 0 Then
            Err.Raise(vbObjectError + 1, "cSky.DeltaM", "DeltaM not valid: " & DeltaM)
        End If
        Exit Function
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Function
    Friend Sub GetX(ByVal dblFix As Double, ByVal intMode As Short, ByVal dblZ As Double, ByVal dblZR As Double, ByRef dblXA As Double, ByRef dblXOz As Double, ByRef dblXG As Double, ByRef blnValid As Boolean)
        'Get path lengths in atmospheres

        On Error GoTo EH
        blnValid = True
        Select Case intMode
            Case 0 'Shcaefer 1993, as suggested by Hoffman & Kaatz, Yodei Bina, in press
                dblXG = 1 / (cosD(dblZ) + 0.0286 * System.Math.Exp(-10.48 * cosD(dblZ)))
                dblXOz = 1 / System.Math.Sqrt(1 - 0.993758 * Square(sinD(dblZ)))
                dblXA = 1 / (cosD(dblZ) + 0.0122 * System.Math.Exp(-24.49 * cosD(dblZ)))
            Case 1 'according to B. E. Schaefer, Sky Telesc., May 1998, 57.
                dblXG = 1 / (cosD(dblZ) + 0.0286 * System.Math.Exp(-10.5 * cosD(dblZ)))
                dblXOz = 1 / System.Math.Sqrt(1 - Square(sinD(dblZ) / (1 + 20 / 6378)))
                dblXA = 1 / (cosD(dblZ) + 0.0123 * System.Math.Exp(-24.5 * cosD(dblZ)))
            Case 2 'Schaefer 1993, as suggested by Hoffman & Kaatz, Yodei Bina, in press
                'Shaefer's equation but use unrefracted angle for Ozone
                dblXG = 1 / (cosD(dblZR) + 0.0286 * System.Math.Exp(-10.48 * cosD(dblZR)))
                dblXOz = 1 / System.Math.Sqrt(1 - 0.993758 * Square(sinD(dblZ)))
                dblXA = System.Math.Abs(1 / (cosD(dblZR) + 0.0122 * System.Math.Exp(-24.49 * cosD(dblZR))))
            Case Else
                Err.Raise(vbObjectError + 1, "cSky.GetX", "intMode not valid: " & intMode)
        End Select
        If dblXG < 0 Then dblXG = 0 'Correction for high above sea-level and negative altitude
        Exit Sub
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Sub
    Friend Sub GetK(ByVal dblFix As Double, ByVal intMode As Short, ByVal dblRA As Double, ByVal dblH As Double, ByRef dblKA As Double, ByRef dblKOz As Double, ByRef dblKR As Double, ByRef dblKW As Double, ByVal dblAltR As Double, ByRef blnValid As Boolean, Optional ByVal blnRedden As Boolean = True)
        'Get extinction coefficients
        Dim dblWa As Double 'Effective wavelength in nm, Gets redder near the horizon
        Dim udtWeatherNow As WeatherNow

        On Error GoTo EH
        blnValid = True
        Call frmWeather.GetWeatherNow(dblFix, udtWeatherNow, blnValid)
        If Not blnValid Then Exit Sub
        Select Case intMode
            Case 0, 2 'Schaefer 1993, as suggested by Hoffman & Kaatz, Yodei Binah, 1, 2001
                'Near the horizon, the light gets redder so the effective wavelength is longer
                If blnRedden Then
                    If dblAltR <= 0.8 Then
                        dblWa = 0.7
                    Else
                        dblWa = 0.55 + 0.15 * System.Math.Exp(-(dblAltR - 0.8) / 8)
                    End If
                Else
                    dblWa = 0.55
                End If
                dblKR = 0.1066 * System.Math.Exp(-udtLocation.dblHeight / 8200) / Square(Square(dblWa / 0.55))
                dblKOz = 0.010333 * (3 + 0.4 * (udtLocation.dblLatitude / RADCON * cosD(dblRA) - cosD(3 * udtLocation.dblLatitude)))
                'Removed factor of 100 in humidity as it seems to be an error (16/4/02)
                'The Ka value now fits the measured values for Jerusalem better
                dblKA = 0.12 * System.Math.Exp(-udtLocation.dblHeight / 1500) * (1 - 0.32 / System.Math.Log(udtWeatherNow.dblHumidity / 100)) ^ 1.3333 * (1 + 0.33 * sinD(dblRA) * System.Math.Sign(udtLocation.dblLatitude)) / ((dblWa / 0.55) ^ 1.33)
                If intMode = 2 Then 'correct for daily variations
                    dblKA = dblKA * (1 + 0.398 * cosD(dblH - 38) * cosD(udtLocation.dblLatitude)) 'was .43
                End If
                dblKW = 0 'Water vapor combined with aerosols by this method
            Case 1 'Schaefer 1998
                dblKR = 0.1066 * System.Math.Exp(-udtLocation.dblHeight / 8200)
                dblKOz = 0.010333 * (3 + 0.4 * (udtLocation.dblLatitude / RADCON * cosD(dblRA) - cosD(3 * udtLocation.dblLatitude)))
                'Added the factor 0.33 which is obviously an error in the published paper                                          here
                dblKA = 0.1 * System.Math.Exp(-udtLocation.dblHeight / 1500) * (1 - 0.32 / System.Math.Log(udtWeatherNow.dblHumidity / 100)) ^ 1.33 * (1 + 0.33 * sinD(dblRA) * System.Math.Sign(udtLocation.dblLatitude))
                dblKW = 0.031 * 0.94 * udtWeatherNow.dblHumidity / 100 * System.Math.Exp(udtWeatherNow.dblTemperature / 15) * System.Math.Exp(-udtLocation.dblHeight / 8200)
            Case Else
                Err.Raise(vbObjectError + 1, "cSky.GetK", "intMode not valid: " & intMode)
        End Select
        Exit Sub
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Sub
    Friend Function BSky(ByVal dblFix As Double, ByVal dblAlt As Double, ByVal dblAz As Double, ByVal intMode As Short, ByRef blnValid As Boolean, Optional ByVal blnFixK As Boolean = False, Optional ByVal dblKaFixed As Double = 0) As Double
        'Calculate the brightness of the sky
        'intMode refers to the model used: 0 - Schaefer 1993
        '                                  1 - B. E. Schaefer, Sky Telesc., May 1998, 57
        '                                  2 - Schaefer 1993 without refraction correction for ozone
        Dim udtMoon As HBody 'Moon's position
        Dim udtSun As HBody 'Sun's position
        'Components of sky brightness
        Dim dblBNight As Double 'Night
        Dim dblBMoon As Double 'From the Moon
        Dim dblBTwi As Double 'Twighlight according to Schaefer
        Dim dblBTwi2 As Double 'Twighlight by interpolating Koomen's data
        Dim dblBDay As Double 'From the Sun
        Dim dblK As Double 'Overall extinction coefficient
        Static dblXM As Double 'Path length in the direction of the Moon
        Static dblXS As Double 'Path length in the direction of the Sun
        Dim dblZM As Double 'Zenith angle of Moon
        Dim dblZS As Double 'Zenith angle of Sun
        Dim dblC4S As Double
        Dim intI As Short
        Dim dblDAZ As Double
        Dim varTemp As Object
        Dim cHBody As New clsHBody
        Dim blnUseSaved As Boolean
        Dim dblC4 As Double
        Dim dblC3 As Double
        Dim dblC2 As Double

        On Error GoTo EH
        blnValid = True
        blnUseSaved = dblFix = udtBSky.dblFixSave And dblAlt = udtBSky.dblAltSave And dblAz = udtBSky.dblAzSave And blnFixK
        If Not blnUseSaved Then
            udtBSky.dblFixSave = dblFix
            udtBSky.dblAltSave = dblAlt
            udtBSky.dblAzSave = dblAz
            Call cHBody.TrueHBody("Moon", dblFix, udtMoon, blnValid)
            If Not blnValid Then Exit Function
            Call cHBody.TrueHBody("Sun", dblFix, udtSun, blnValid)
            If Not blnValid Then Exit Function
            udtBSky.dblMoonMag = cHBody.Magnitude("Moon", udtMoon, dblFix, blnValid)
            If Not blnValid Then Exit Function
            udtBSky.dblSunAlt = udtSun.AltTopo
            Call GetK(dblFix, intMode, udtSun.RATopo, udtSun.HourAngTopo, udtBSky.dblKA, udtBSky.dblKOz, udtBSky.dblKR, udtBSky.dblKW, dblAlt, blnValid, False)
            If Not blnValid Then Exit Function
            udtBSky.dblZ = 90 - dblAlt
        End If
        If blnFixK Then udtBSky.dblKA = dblKaFixed
        dblK = udtBSky.dblKA + udtBSky.dblKR + udtBSky.dblKOz + udtBSky.dblKW
        If Not blnUseSaved Then
            'Find angular separation from the Moon
            udtBSky.dblRhoM = sinD(udtMoon.AltRefr) * sinD(dblAlt) + cosD(udtMoon.AltRefr) * cosD(dblAlt) * cosD(udtMoon.AzTopo - dblAz)
            If udtBSky.dblRhoM > 0.999995 Then
                udtBSky.dblRhoM = System.Math.Sqrt(Square(dblAz - udtMoon.AzTopo) * cosD((udtMoon.AltRefr + dblAlt) / 2) + Square(udtMoon.AltRefr - dblAlt))
            ElseIf udtBSky.dblRhoM <= -1 Then
                udtBSky.dblRhoM = 180
            Else
                udtBSky.dblRhoM = acosD(udtBSky.dblRhoM)
            End If
            If udtBSky.dblRhoM < udtMoon.Phase * 2 * udtMoon.Radius * 0.97 Then udtBSky.dblRhoM = udtMoon.Phase * 2 * udtMoon.Radius * 0.97 'An approximation for the smallest effective distance
            'Find angular separation from the Sun
            udtBSky.dblRhoS = sinD(udtSun.AltRefr) * sinD(dblAlt) + cosD(udtSun.AltRefr) * cosD(dblAlt) * cosD(udtSun.AzTopo - dblAz)
            If udtBSky.dblRhoS > 0.999995 Then
                udtBSky.dblRhoS = System.Math.Sqrt(Square(dblAz - udtSun.AzTopo) * cosD((udtSun.AltRefr + dblAlt) / 2) + Square(udtSun.AltRefr - dblAlt))
            ElseIf udtBSky.dblRhoS <= -1 Then
                udtBSky.dblRhoS = 180
            Else
                udtBSky.dblRhoS = acosD(udtBSky.dblRhoS)
            End If
            If udtBSky.dblRhoS < 2 * udtSun.Radius * 0.97 Then udtBSky.dblRhoS = 2 * udtSun.Radius * 0.97
        End If
        Select Case intMode
            Case 0 'Schaefer 1993, as suggested by Hoffman & Kaatz, Yodei Binah, 1, 2001
                If Not blnUseSaved Then
                    udtBSky.dblX = 1 / (cosD(udtBSky.dblZ) + 0.025 * System.Math.Exp(-11 * cosD(udtBSky.dblZ)))
                    If udtMoon.RefrValid And udtMoon.AltRefr > 0 Then
                        udtBSky.dblXM = 1 / (cosD(90 - udtMoon.AltRefr) + 0.025 * System.Math.Exp(-11 * cosD(90 - udtMoon.AltRefr)))
                    Else
                        udtBSky.dblXM = 40
                    End If
                    If udtSun.RefrValid And udtSun.AltRefr > 0 Then
                        udtBSky.dblXS = 1 / (cosD(90 - udtSun.AltRefr) + 0.025 * System.Math.Exp(-11 * cosD(90 - udtSun.AltRefr)))
                    Else
                        udtBSky.dblXS = 40
                    End If
                End If
                dblBNight = 90.09 * (1 + 0.3 * System.Math.Cos(6.283 * (dblFix / 365.242 - 1991) / 11.2)) * (0.4 + 0.6 / System.Math.Sqrt(1 - 0.96 * Square(sinD(udtBSky.dblZ)))) * 10 ^ (-0.4 * dblK * udtBSky.dblX)
                'Find angular separation from the Moon
                If Not blnUseSaved Then
                    udtBSky.dblRhoM = 10 ^ 5.36 * (1.06 + Square(cosD(udtBSky.dblRhoM))) + 10 ^ (6.15 - udtBSky.dblRhoM / 40) + 62000000.0# / Square(udtBSky.dblRhoM)
                    udtBSky.dblRhoS2 = udtBSky.dblRhoS
                    udtBSky.dblRhoS = 10 ^ 5.36 * (1.06 + Square(cosD(udtBSky.dblRhoS))) + 10 ^ (6.15 - (udtBSky.dblRhoS / 40)) + 62000000.0# / Square(udtBSky.dblRhoS)
                End If
                dblBMoon = udtBSky.dblRhoM * 10 ^ (-0.4 * (udtBSky.dblMoonMag + 16.57 + dblK * udtBSky.dblXM)) * (1 - 10 ^ (-0.4 * dblK * udtBSky.dblX))
                dblBTwi = 10 ^ ((udtBSky.dblRhoS / 90) - 1.1)
                If dblBTwi < 1 Then dblBTwi = 1
                dblBTwi = dblBTwi * 10 ^ (8.45 + 0.4 * udtBSky.dblSunAlt) * (1 - 10 ^ (-0.4 * dblK * udtBSky.dblX))
                dblBDay = 11700 * udtBSky.dblRhoS * 10 ^ (-0.4 * dblK * dblXS) * (1 - 10 ^ (-0.4 * dblK * udtBSky.dblX))
                If dblBTwi < dblBDay Then
                    BSky = dblBTwi
                Else
                    BSky = dblBDay
                End If
                BSky = (BSky + dblBNight + dblBMoon) * 0.00000318
            Case 1 'Schaefer 1998
                If Not blnUseSaved Then
                    udtBSky.dblX = 1 / (cosD(udtBSky.dblZ) + 0.025 * System.Math.Exp(-11 * cosD(udtBSky.dblZ)))
                    If Not udtMoon.RefrValid Or udtMoon.AltRefr < 0 Then
                        udtBSky.dblXM = 40
                    Else
                        dblZM = 90 - udtMoon.AltRefr
                        udtBSky.dblXM = 1 / (cosD(dblZM) + 0.025 * System.Math.Exp(-11 * cosD(dblZM)))
                    End If
                    If Not udtSun.RefrValid Or udtSun.AltRefr < 0 Then
                        udtBSky.dblXS = 40
                    Else
                        dblZS = 90 - udtSun.AltRefr
                        udtBSky.dblXS = 1 / (cosD(dblZS) + 0.025 * System.Math.Exp(-11 * cosD(dblZS)))
                    End If
                End If
                dblBNight = 0.0000000000001 * (1 + 0.3 * System.Math.Cos(6.283 * (dblFix / 365.242 - 1991) / 11.2)) * (0.4 + 0.6 / System.Math.Sqrt(1 - 0.96 * Square(sinD(udtBSky.dblZ)))) * 10 ^ (-0.4 * dblK * udtBSky.dblX)
                If Not blnUseSaved Then
                    udtBSky.dblRhoM = 10 ^ 5.36 * (1.06 + Square(cosD(udtBSky.dblRhoM))) + 10 ^ (6.15 - udtBSky.dblRhoM / 40) + 62000000.0# / Square(udtBSky.dblRhoM)
                    udtBSky.dblRhoS2 = udtBSky.dblRhoS
                    udtBSky.dblRhoS = 10 ^ 5.36 * (1.06 + Square(cosD(udtBSky.dblRhoS))) + 10 ^ (6.15 - (udtBSky.dblRhoS / 40)) + 62000000.0# / Square(udtBSky.dblRhoS)
                End If
                dblC4S = 10 ^ (-0.4 * dblK / dblXM)
                dblBMoon = 10 ^ (-0.4 * (udtBSky.dblMoonMag + 11.05 + 43.27)) * (1 - 10 ^ (-0.4 * dblK * udtBSky.dblX)) * (udtBSky.dblRhoM * dblC4S + 440000 * (1 - dblC4S))
                dblBTwi = 10 ^ (-0.4 * (-26.74 + 11.05 + 32.5 - udtBSky.dblSunAlt - udtBSky.dblZ / 360 / dblK)) * 100 / udtBSky.dblRhoS * (1 - 10 ^ (-0.4 * dblK))
                dblC4S = 10 ^ (-0.4 * dblK / dblXS)
                dblBDay = 10 ^ (-0.4 * (-26.74 + 11.05 + 43.27)) * (1 - 10 ^ (-0.4 * dblK * udtBSky.dblX)) * (udtBSky.dblRhoS * dblC4S + 440000 * (1 - dblC4S))
                If dblBTwi > dblBDay Then
                    BSky = dblBDay + dblBNight
                Else
                    BSky = dblBTwi + dblBNight
                End If
                If udtMoon.RefrValid And udtMoon.AltRefr >= 0 Then
                    BSky = BSky + dblBMoon
                End If
                BSky = BSky / 0.00000000000000111 * 0.00000318
            Case 2 'Hoffman & Kaatz in Yodei Binah, 1, 2001 when Sun's altitude between -15 and 5 and more than a degree from the Sun, Schaefer 1993 otherwise
                If Not blnUseSaved Then
                    If udtBSky.dblZ > 90 Then
                        udtBSky.dblX = 40
                    Else
                        udtBSky.dblX = System.Math.Abs(1 / (cosD(udtBSky.dblZ) + 0.025 * System.Math.Exp(-11 * cosD(udtBSky.dblZ))))
                        If udtBSky.dblX > 40 Then udtBSky.dblX = 40
                    End If
                    If udtMoon.RefrValid And udtMoon.AltRefr > 0 Then
                        udtBSky.dblXM = 1 / (cosD(90 - udtMoon.AltRefr) + 0.025 * System.Math.Exp(-11 * cosD(90 - udtMoon.AltRefr)))
                        If udtBSky.dblXM > 40 Then udtBSky.dblXM = 40
                    Else
                        udtBSky.dblXM = 40
                    End If
                    If udtSun.RefrValid And udtSun.AltRefr > 0 Then
                        udtBSky.dblXS = 1 / (cosD(90 - udtSun.AltRefr) + 0.025 * System.Math.Exp(-11 * cosD(90 - udtSun.AltRefr)))
                        If udtBSky.dblXS > 40 Then udtBSky.dblXS = 40
                    Else
                        udtBSky.dblXS = 40
                    End If
                End If
                If Not blnUseSaved Then
                    udtBSky.dblBNightBase = 0.0002865 * (1 + 0.3 * System.Math.Cos(TwoPi * (dblFix / 365.242 - 2000.21) / 11.2)) * (0.4 + 0.6 / System.Math.Sqrt(1 - 0.96 * Square(sinD(udtBSky.dblZ))))
                    'Find angular separation from the Moon
                    udtBSky.dblRhoM = 10 ^ 5.36 * (1.06 + Square(cosD(udtBSky.dblRhoM))) + 10 ^ (6.15 - udtBSky.dblRhoM / 40) + 62000000.0# / Square(udtBSky.dblRhoM)
                    udtBSky.dblRhoS2 = udtBSky.dblRhoS
                    udtBSky.dblRhoS = 10 ^ 5.36 * (1.06 + Square(cosD(udtBSky.dblRhoS2))) + 10 ^ (6.15 - (udtBSky.dblRhoS2 / 40)) + 62000000.0# / Square(udtBSky.dblRhoS2)
                    udtBSky.dblBMoonBase = 0.00000318 * udtBSky.dblRhoM
                    udtBSky.dblBTwiBase = 541.4 * 100 / udtBSky.dblRhoS2 * 10 ^ (0.4 * udtSun.AltTopo)
                    udtBSky.dblBDayBase = 0.02664 * 1.9 '0.0372 '0.02664
                End If
                dblC2 = -0.4 * dblK
                dblC3 = 10 ^ (dblC2 * udtBSky.dblX)
                dblBNight = udtBSky.dblBNightBase * dblC3
                dblBMoon = udtBSky.dblBMoonBase * 10 ^ (-0.4 * (udtBSky.dblMoonMag + 16.57 + dblK * udtBSky.dblXM)) * (1 - dblC3)
                dblBTwi = udtBSky.dblBTwiBase * (1 - 10 ^ (-0.4 * dblK)) * 10 ^ (0.4 * udtBSky.dblZ / 360 / dblK)
                dblC4 = 10 ^ (dblC2 * udtBSky.dblXS)
                dblBDay = udtBSky.dblBDayBase * (udtBSky.dblRhoS * dblC4 + 440000 * (1 - dblC4)) * (1 - 10 ^ (dblC2 * udtBSky.dblX))
                If dblBTwi < dblBDay Then
                    BSky = dblBTwi
                Else
                    BSky = dblBDay
                End If
                BSky = (BSky + dblBNight + dblBMoon)
        End Select
        Exit Function
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Function
    Friend Sub spline(ByVal dblX() As Double, ByVal dblY() As Double, ByRef dblY2() As Double, ByVal intN As Short)
        'For a set of intN points (dblX, dblY) compute the 2nd derivatives (dblY2) for a natural spline
        Dim dblU() As Double 'Work space
        Dim intI As Short 'Loop counter
        Dim dblSig As Double
        Dim dblP As Double

        ReDim dblU(intN)
        dblY2(0) = 0
        dblU(0) = 0
        For intI = 1 To intN - 2
            dblSig = (dblX(intI) - dblX(intI - 1)) / (dblX(intI + 1) - dblX(intI - 1))
            dblP = dblSig * dblY2(intI - 1) + 2
            dblY2(intI) = (dblSig - 1) / dblP
            dblU(intI) = (dblY(intI + 1) - dblY(intI)) / (dblX(intI + 1) - dblX(intI)) - (dblY(intI) - dblY(intI - 1)) / (dblX(intI) - dblX(intI - 1))
            dblU(intI) = (6 * dblU(intI) / (dblX(intI + 1) - dblX(intI - 1)) - dblSig * dblU(intI - 1)) / dblP
        Next
        If intN > 0 Then dblY2(intN - 1) = 0
        For intI = intN - 2 To 0 Step -1
            dblY2(intI) = dblY2(intI) * dblY2(intI + 1) + dblU(intI)
        Next
    End Sub
    Friend Function splint(ByVal dblX() As Double, ByVal dblY() As Double, ByVal dblY2() As Double, ByVal intN As Short, ByVal dblXGiven As Double) As Double
        'Spline interpolation of a table of values
        Dim intKLo As Short
        Dim intKHi As Short
        Dim intK As Short
        Dim dblH As Double
        Dim dblB As Double
        Dim dblA As Double

        'Find the points that bracket dblXGiven
        intKLo = 0
        intKHi = intN - 1
        While intKHi - intKLo > 1
            intK = Int((intKHi + intKLo) / 2)
            If dblX(intK) > dblXGiven Then
                intKHi = intK
            Else
                intKLo = intK
            End If
        End While
        If intKHi < 0 Or intKLo < 0 Then
            MsgBox("Bad intKHi and intKLo array in SPLINT.")
            splint = dblX(0)
        Else
            dblH = dblX(intKHi) - dblX(intKLo)
            If dblH = 0 Then
                MsgBox("Bad dblX array in SPLINT.")
                splint = dblX(intKHi)
            Else
                dblA = (dblX(intKHi) - dblXGiven) / dblH
                dblB = (dblXGiven - dblX(intKLo)) / dblH
                splint = dblA * dblY(intKLo) + dblB * dblY(intKHi) + ((dblA * dblA * dblA - dblA) * dblY2(intKLo) + (dblB * dblB * dblB - dblB) * dblY2(intKHi)) * dblH * dblH / 6
            End If
        End If
    End Function
    Function ProbVis(ByVal dblFix As Double, ByVal dblAlt As Double, ByVal dblAz As Double, ByVal dblFovealMag As Double, ByVal dblPeriphMag As Double, ByVal dblKA As Double, ByVal dblXA As Double, ByRef blnValid As Boolean) As Double
        'Calculate the probability of visibility from photometric paramters
        'Take into account the shape of the object
        'Input parameters    :
        '       dblFovealMag    Apparent stellar magnitude of the object in the Ricco area (within 6.5' of center)
        '       dblPeriphMag    Apparent stellar magnitude of the object outside the Ricco area (more than 6.5' of center)
        '       dblKA           absorption constant due to aerosols
        '       dblXA           aerosol path length (atmospheres)
        Dim dblKaTrial As Double 'Trial value of aerosol absorbtion
        Dim dblBSky As Double 'Background brightness (Cd/m^2)
        Dim dblFovealIlluminance As Double 'Illuminance in the foveal region
        Dim dblPeriphIlluminance As Double 'Illuminance outside the foveal region
        Dim dblIT As Double 'Threshold effective illuminance
        Dim dblIEff As Double 'Effective illuminance
        Dim dblCT As Double 'Threshold contrast
        Dim dblI2 As Double 'Upper limit of illuminance to be considered in integration
        Dim intI As Short 'Loop counters for integration
        Dim intJ As Short
        Dim dblP As Double 'Probability scaling factor
        Dim dblP2 As Double 'Probability scaling factor
        Dim dblP2Old As Double
        Dim dblP3 As Double 'Probability scaling factor
        Dim dblN As Double 'Normalization constants for integration
        Dim dblN2 As Double
        Dim dblLogB As Double 'log10 of backround brightness
        Dim dblMaxI As Double 'Highest contrast considered
        Dim dblStep As Double 'Integration intervals
        Dim dblStep2 As Double
        Dim dblSig As Double
        Dim dblTemp As Double
        Dim dblTemp2 As Double
        Dim dblTemp3 As Double
        Dim dblMagEff As Double 'Effective magnitude
        Const KaFactor As Double = 0.7842 'Was 2.48
        Const SCurve As Double = 0.5 ' 0.32 Adjustments to fit straight line
        Const Height As Double = 0.3 '1.201
        Const SCurveP As Double = 0.65 ' 0.32 Adjustments to fit straight line
        Const HeightP As Double = 1.0# '1.201
        Dim SigFactor As Double
        Dim ITFactor As Double
        Dim dblKaFactor As Double
        Dim dblK As Double
        Dim dblDiffMag As Double
        Dim dblPBase As Double

        On Error GoTo EH
        blnValid = True
        dblP = 0
        dblP2 = 0
        Select Case intNIntegrals
            Case 16
                dblKaFactor = 5
            Case 32
                dblKaFactor = 6.4
            Case 64
                dblKaFactor = 8
            Case 128
                dblKaFactor = 9.9
            Case Else
                Err.Raise(vbObjectError + 1, "cSky.ProbVis", "Invalid intNIntegrals in ProbVis :" & intNIntegrals)
        End Select
        dblK = 1.267
        For intI = 1 To intNIntegrals
            dblKaTrial = intI * dblKA * dblKaFactor / intNIntegrals
            'Logarithm of the background brightness
            dblBSky = BSky(dblFix, dblAlt, dblAz, 2, blnValid, True, dblKaTrial)
            If Not blnValid Then Exit Function
            dblLogB = Log10(dblBSky)
            'Calculate the illuminance, the illuminance used by the eye and the brighness of the object
            'Foveal illuminance (within 6.5' of center of vision)
            'Start with log illuminance
            dblDiffMag = (dblKA - dblKaTrial) * dblXA - 16.57
            dblFovealIlluminance = (-dblFovealMag + dblDiffMag) / 2.5
            If dblFovealIlluminance > 80 Then 'If out of range make it a large value
                dblFovealIlluminance = 1.0E+80
            Else 'otherwise calculate the illuminance
                dblFovealIlluminance = 10 ^ (dblFovealIlluminance)
            End If
            'Peripheral illuminance (more than 6.5' from center of vision)
            dblPeriphIlluminance = (-dblPeriphMag + dblDiffMag) / 2.5
            If dblPeriphIlluminance > 80 Then
                dblPeriphIlluminance = 1.0E+80
            Else
                dblPeriphIlluminance = 10 ^ (dblPeriphIlluminance)
            End If
            'Model from Hecht
            If dblLogB < -2.3172 Then
                dblIT = 10 ^ -9.8 * Square(1 + System.Math.Sqrt(10 ^ (-1.9 + dblLogB + 5.4972)))
            Else
                dblIT = 10 ^ -8.35 * Square(1 + System.Math.Sqrt(10 ^ (-5.9 + dblLogB + 5.4972)))
            End If
            dblIEff = dblFovealIlluminance + dblPeriphIlluminance * (0.1278 + dblLogB * (-0.0533 + dblLogB * 0.0049))
            If dblIEff < dblFovealIlluminance Then dblIEff = dblFovealIlluminance
            dblPBase = System.Math.Exp(-Square(System.Math.Log(dblKaTrial) - System.Math.Log(dblKA) - 0.3698) / 1.4792) / dblKaTrial * 0.4639
            dblP = dblP + dblPBase * Square(1 - System.Math.Exp(-dblK * Square(dblIEff / dblIT)))
            dblP2 = dblP2 + dblPBase
        Next
        dblP = dblP * dblKA * dblKaFactor / intNIntegrals
        dblP2 = dblP2 * dblKA * dblKaFactor / intNIntegrals
        dblP = dblP / dblP2
        If dblPeriphMag >= 10 Then
            dblP = Norm(1.09 * (NormInv(dblP, blnValid) + 0.06), blnValid)
            If Not blnValid Then Exit Function
        Else
            dblP = Norm(1.339239 * (NormInv(dblP, blnValid) - 0.32848), blnValid)
            If Not blnValid Then Exit Function
        End If
        If Not udtObserver.blnOverall Then
            dblP = Norm((NormInv(dblP, blnValid) + udtObserver.dblLevel) / 0.86, blnValid)
            If Not blnValid Then Exit Function
        End If
        ProbVis = dblP
        '    End If
        Exit Function
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Function
    Function UsefulWidth(ByVal dblBSky As Double) As Double
        'The effective width is related to the eye's resolution as a function of backgroud brightness
        If dblBSky > 10 ^ (-0.3283 + 0.5) Then
            UsefulWidth = 0.01111111 * 10 ^ (8.28 * (dblBSky * 3145.6) ^ -0.29)
        ElseIf dblBSky > 10 ^ (-1.3283 + 0.5) Then
            UsefulWidth = 0.01111111 * 10 ^ (8.28 * (dblBSky * 3145.6) ^ -0.29) * (Log10(dblBSky) + 1.3283 - 0.5) + 0.105 * 10 ^ (0.3 * (dblBSky * 3145.6) ^ -0.29) / 2.5 * (-0.3283 - Log10(dblBSky) + 0.5)
        Else
            UsefulWidth = 0.105 * 10 ^ (0.3 * (dblBSky * 3145.6) ^ -0.29) / 2.5 '2 is a correction factor because the dark probability was underestimated'1.475
            If UsefulWidth > 0.25 Then UsefulWidth = 0.25
        End If
    End Function
    Friend Function SeaLevelHorizon(ByVal udtWeatherNow As WeatherNow, ByRef blnValid As Boolean) As Double
        'Calculate apparent angle of dip to the sea
        Dim dblPSL As Double
        Dim dblTSL As Double
        Dim dblMu0 As Double
        Dim dblmu As Double
        Dim dblHeight As Double 'The height above sea level but set to zero if below seal level

        On Error GoTo EH
        blnValid = True
        If udtWeatherNow.dblPressure / System.Math.Exp(udtLocation.dblHeight / 7143) > PMax Then
            dblHeight = 0 'Cannot calculate for unrealistically high pressures
            dblPSL = 1
            dblTSL = 1
            dblmu = 1.00029329
            dblMu0 = 1.00029329
        Else
            dblPSL = udtWeatherNow.dblPressure * System.Math.Exp(udtLocation.dblHeight / 7143) / 1013.25
            dblTSL = (udtWeatherNow.dblTemperature - 0.005688 * udtLocation.dblHeight + 273.15) / 273.15
            dblmu = 1 + 0.00029329 * 273.15 / (273.15 + udtWeatherNow.dblTemperature) * udtWeatherNow.dblPressure / 1013.25
            dblMu0 = 1 + 0.00029329 * 273.15 / (273.15 + udtWeatherNow.dblTemperature + 0.005688 * udtLocation.dblHeight) * udtWeatherNow.dblPressure * System.Math.Exp(udtLocation.dblHeight / 7143) / 1013.25
            If udtLocation.dblHeight < 0 Then
                dblHeight = 0 'Cannot calculate for below sea level
            Else
                dblHeight = udtLocation.dblHeight
            End If
        End If
        '        SeaLevelHorizon = -1.11 * Sqr(udtLocation.dblHeight / 1000) * dblPSL / dblTSL
        'Schaefer's equation
        'SeaLevelHorizon = -0.604 * dblPSL / dblTSL - acosD(1 / (1 + udtLocation.dblHeight / 6378000))
        'John Sweer, "The Path of a Ray of Light Tangent to the Surface of the Earth", Journal of the Optical Society of America, 28 (September 1938):32729
        SeaLevelHorizon = -0.604 * dblPSL / dblTSL - acosD(6371000.0 * dblMu0 / dblmu / (6371000.0 + dblHeight))
        If SeaLevelHorizon > 0 Then
            SeaLevelHorizon = 0 'Problems with extreme heights
        ElseIf SeaLevelHorizon < -45 Then
            SeaLevelHorizon = 0
        End If
        If SeaLevelHorizon < -90 Or SeaLevelHorizon > 0 Then
            Err.Raise(vbObjectError + 1, "cSky.SeaLevelHorizon", "Invalid SeaLevelHorizon: " & SeaLevelHorizon)
        End If
        Exit Function
EH:
        If Err.Number = vbObjectError + 1 Then
            MsgBox("Error in " & Err.Source & ", " & Err.Description)
            blnValid = False
        Else
            Debug.Print(VB6.TabLayout(Err.Number, Err.Description))
            Err.Raise(Err.Number)
        End If
    End Function
End Class